@@ -13,7 +13,7 @@ formatter = logging.Formatter("%(asctime)s:%(name)s:%(levelname)s: %(message)s")
 
 LOG_DIR = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), 'logs')
 os.makedirs(LOG_DIR, exist_ok=True)
-file_handler = logging.FileHandler(os.path.join(LOG_DIR, "General_Tools.log"))
+file_handler = logging.FileHandler(os.path.join(LOG_DIR, "general_tools.log"))
 file_handler.setFormatter(formatter)
 
 logger.addHandler(file_handler)
